package org.bitbucket.bradleysmithllc.etlunit.feature.mysql_database;

import com.mysql.jdbc.Driver;
import org.bitbucket.bradleysmithllc.etlunit.feature.database.BaseDatabaseImplemenation;
import org.bitbucket.bradleysmithllc.etlunit.feature.database.DatabaseConnection;
import org.bitbucket.bradleysmithllc.etlunit.feature.database.NullResultSetClient;
import org.bitbucket.bradleysmithllc.etlunit.util.IOUtils;
import org.bitbucket.bradleysmithllc.etlunit.util.VelocityUtil;

import java.net.URL;
import java.sql.Connection;
import java.sql.ResultSet;
import java.util.HashMap;
import java.util.Map;

public class MySQLDatabaseImplementation extends BaseDatabaseImplemenation {
	public String getImplementationId() {
		return "mysql";
	}

	public Object processOperationSub(operation op, OperationRequest request) throws UnsupportedOperationException {
		switch (op) {
			case createDatabase: {
				InitializeRequest initializeRequest = request.getInitializeRequest();
				try {
					executeScript(
							IOUtils.readURLToString(MySQLDatabaseImplementation.class.getResource("/createDatabase.vm")),
							initializeRequest.getConnection(),
							initializeRequest.getMode(),
							SYSADMIN
					);
				} catch (Exception testExecutionError) {
					throw new RuntimeException("Error initializing database", testExecutionError);
				}
				break;
			}
			case preCreateSchema:
				break;
			case postCreateSchema:
				break;
			case completeDatabaseInitialization:
				break;
			case materializeViews: {
				InitializeRequest initializeRequest = request.getInitializeRequest();
				try {
					URL url = getClass().getResource("/materializeView.vm");
					final String materialize = IOUtils.readURLToString(url);

					final Map<String, String> context = new HashMap<String, String>();
					// grab the renaming strategy for views
					final DatabaseConnection dc = initializeRequest.getConnection();

					String renStrat = dc.getDatabaseProperties().get("view-rename-strategy");
					if (renStrat == null || renStrat.trim().equals(""))
					{
						renStrat = "_V";
					}

					context.put("view-rename-strategy", renStrat);

					jdbcClient.useResultSetScriptResource(
							dc,
							initializeRequest.getMode(),
							new NullResultSetClient() {
								@Override
								public void next(Connection conn, ResultSet st, DatabaseConnection connection, String mode, int id) throws Exception {
									// issue a rename for the view
									String viewName = st.getString(1);
									applicationLog.info("Found View [" + viewName + "]");
									context.put("view-name", viewName);

									jdbcClient.useResultSetScript(connection, mode, null, VelocityUtil.writeTemplate(materialize, context), id);
								}
							},
							"selectViews.vm"
					);
				} catch (Exception testExecutionError) {
					throw new RuntimeException("Error initializing database", testExecutionError);
				}
				break;
			}
			case dropConstraints:
				break;
			case preCleanTables:
				break;
			case postCleanTables:
				break;
		}

		return null;
	}

	@Override
	public String getJdbcUrl(DatabaseConnection dc, String mode, int id) {
		if (id == DATABASE) {
			return "jdbc:mysql://" + dc.getServerName() + "/" + getDatabaseName(dc, mode);
		} else {
			return "jdbc:mysql://" + dc.getServerName() + "/";
		}
	}

	/**
	 * MySQL only handles 64-character database names.
	 *
	 * @param dc
	 * @param mode
	 * @return
	 */
	@Override
	public String getDatabaseName(DatabaseConnection dc, String mode) {
		return IOUtils.hashToLength(super.getDatabaseName(dc, mode), 64).toLowerCase();
	}

	/**
	 * Defer to the login name for the password
	 *
	 * @param dc
	 * @param mode
	 * @return
	 */
	@Override
	protected String getPasswordImpl(DatabaseConnection dc, String mode) {
		return getLoginNameImpl(dc, mode);
	}

	/**
	 * Mysql limits users to 16 characters.  Defer to the project UID
	 *
	 * @param dc
	 * @param mode
	 * @return
	 */
	@Override
	protected String getLoginNameImpl(DatabaseConnection dc, String mode) {
		return IOUtils.hashToLength(getDatabaseName(dc, mode), 16);
	}

	@Override
	public Class getJdbcDriverClass() {
		return Driver.class;
	}

	public String getDefaultSchema(DatabaseConnection databaseConnection, String mode) {
		return getDatabaseName(databaseConnection, mode);
	}

	@Override
	public String escapeIdentifier(String name) {
		return "`" + name + "`";
	}
}
